//+------------------------------------------------------------------+
//|                                           AsymmetricFractals.mq4 |
//|                               Copyright  2012, Gehtsoft USA LLC |
//|                                            http://fxcodebase.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2012, Gehtsoft USA LLC"
#property link      "http://fxcodebase.com"

//+------------------------------------------------------------------+
//{ Generic code                                                     |
//+------------------------------------------------------------------+
//{ Typedefs
#define  PRICE double            // A Price
#define     PRICE_MAX   (PRICE)EMPTY_VALUE
#define     PRICE_MIN   (PRICE)0.0
#define     CHANGE      PRICE    // Difference of two prices.
#define  INDEX    uint           // Zero based.
   #define  INV_INDEX      UINT_MAX
#define  COUNT    uint           //  One based.
   #define  INV_COUNT      UINT_MAX
#define  MONEY    double
#define  SYMBOL   string
//} Typedefs
//+------------------------------------------------------------------+
//} Generic code                                                     |
//{ Specific code                                                    |
//+------------------------------------------------------------------+
//---- indicator parameters
extern COUNT   FramesBefore=4;
extern COUNT   FramesAfter=4;

//---- indicator buffers
#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1 Red
#property indicator_color2 Blue
double ExtUpFractalsBuffer[];
double ExtDownFractalsBuffer[];

bool		gTimer;
int OnInit(){	gTimer=EventSetTimer(1);
    SetIndexBuffer(0,ExtUpFractalsBuffer);
    SetIndexBuffer(1,ExtDownFractalsBuffer);
    SetIndexStyle(0,DRAW_ARROW,0,4);
    SetIndexArrow(0,119);
    SetIndexStyle(1,DRAW_ARROW,0,4);
    SetIndexArrow(1,119);
// SetIndexEmptyValue(0,0.0);
// SetIndexEmptyValue(1,0.0);
    SetIndexLabel(0,"Fractal Up");
    SetIndexLabel(1,"Fractal Down");

   return INIT_SUCCEEDED;
}  // OnInit

void OnDeinit(const int reason){	EventKillTimer(); }

//+------------------------------------------------------------------+
//| Custom indicator timer function                                  |
//+------------------------------------------------------------------+
double   gChartScale=0;
void OnTimer(void){
   double   csPrev   = gChartScale;
      int      chp   = ChartGetInteger(0, CHART_HEIGHT_IN_PIXELS);
                                                      if(chp == 0)   return;
   gChartScale = ( ChartGetDouble(0, CHART_PRICE_MAX)
                 - ChartGetDouble(0, CHART_PRICE_MIN)
                 ) / chp;
   if(gChartScale != csPrev){
         static int  NAspread[];    static long NAvolume[];
         /*(void)*/OnCalculate(Bars,         // rates_total,
                               0,            // prev_calculated,
                               Time, Open, High, Low, Close,
                               NAvolume,     // NU tick_volume[],
                               NAvolume,     // NU volume[],
                               NAspread);    // NU spread[]
   }
}  // OnTimer
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const PRICE  &open[],
                const PRICE  &high[],
                const PRICE  &low[],
                const PRICE  &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[]){
// EventSetTimer sometimes fails in OnInit, on MT4, with internal error. So
// remember to try it in OnCalculate
if (!gTimer){	gTimer=EventSetTimer(1);	if (!gTimer) return prev_calculated; }
   #define  LAST              0                       // Draw bar zero?
   #define  REDRAW_BAR_LAST   (LAST == 0)             // Redraw bar last.
   #define  LOOKBACK          int(FramesBefore+FramesAfter)
   int      iBar        = rates_total - MathMax(LOOKBACK, prev_calculated);
   while(iBar > LAST){  --iBar;
      bool FrUp=true;
      bool FrDn=true;
      int ii;
      for (ii=1;ii<=(int)FramesBefore;ii++)
      {
       if (High[iBar+FramesAfter+ii]>=High[iBar+FramesAfter]) FrUp=false;
       if (Low[iBar+FramesAfter+ii]<=Low[iBar+FramesAfter]) FrDn=false;
      }
      for (ii=1;ii<=(int)FramesAfter;ii++)
      {
       if (High[iBar+FramesAfter-ii]>=High[iBar+FramesAfter]) FrUp=false;
       if (Low[iBar+FramesAfter-ii]<=Low[iBar+FramesAfter]) FrDn=false;
      }

      if (FrUp)
        {
         ExtUpFractalsBuffer[iBar+FramesAfter]=High[iBar+FramesAfter]
                                                         + 6 * gChartScale;
        }
        else
        {
         ExtUpFractalsBuffer[iBar+FramesAfter]=EMPTY_VALUE; // EMPTY;
        }
      if (FrDn)
        {
         ExtDownFractalsBuffer[iBar+FramesAfter]=Low[iBar+FramesAfter]
                                                         - 8 * gChartScale;
        }
        else
        {
         ExtDownFractalsBuffer[iBar+FramesAfter]=EMPTY_VALUE;  //
        }
     }
   //----
   return rates_total-LAST-REDRAW_BAR_LAST;
}  // OnCalculate
#ifndef ALERT_ID
#define ALERT_ID
	void		alert_id(string msg){
		Alert(StringFormat("%s %s %s %s", WindowExpertName(), _Symbol,
													 as_string(PERIOD_CURRENT), msg) );
	}
void     print_id(string msg){
   PrintFormat(       "%s %s %s %s", WindowExpertName(), _Symbol,
                                     as_string(PERIOD_CURRENT), msg);
}
string   as_string(ENUM_TIMEFRAMES period){
   if(period == PERIOD_CURRENT)  period   = (ENUM_TIMEFRAMES) _Period;
   string   period_xxx  = EnumToString(period);                // PERIOD_XXX
   return StringSubstr(period_xxx, 7);                         // XXX
}
#endif
//+------------------------------------------------------------------+
